/**
 * Handles the database
 */
package com.kvnallsn.militarynews;

import java.util.ArrayList;
import java.util.List;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

/**
 * @author Kevin Allison
 *
 */
public class DatabaseLayer {

	// Constant Variables used for the database
	public static final String KEY_TITLE = "title";
	public static final String KEY_LINK = "link";
	public static final String KEY_DATE = "date";
	public static final String KEY_LINK_MORE = "more";
	public static final String KEY_IMAGE = "image";
	public static final String KEY_USER = "user";
	public static final String KEY_ROWID = "_id";
	
	// Actual database and helper
	private DatabaseHelper dbHelper;
	private SQLiteDatabase db;
	private Context mCtx = null;
	private boolean dbOpen = false;
	
	//Other useful database constants
	public static final String DATABASE_NAME = "milnews_db";
	public static final String NEWS_TABLE = "newstable";
	public static final String TWITTER_TABLE = "twittertable";
	public static final String BLOG_TABLE = "blogtable";
	public static final String VIDEO_TABLE = "videotable";
	public static final int DATABASE_VERSION = 2;
	
	// Class to help with database interaction
	private class DatabaseHelper extends SQLiteOpenHelper {

		public DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(createTableStmt(NEWS_TABLE));
			db.execSQL(createTableStmt(TWITTER_TABLE));
			db.execSQL(createTableStmt(BLOG_TABLE));
			db.execSQL(createTableStmt(VIDEO_TABLE));
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			db.execSQL("DROP TABLE IF EXISTS " + NEWS_TABLE);
			db.execSQL("DROP TABLE IF EXISTS " + TWITTER_TABLE);
			db.execSQL("DROP TABLE IF EXISTS " + BLOG_TABLE);
			db.execSQL("DROP TABLE IF EXISTS " + VIDEO_TABLE);
			onCreate(db);
		}
		
		public String createTableStmt(String tableName) {
			return "create table " + tableName + " ( " 
		    					   + KEY_ROWID + " integer primary key autoincrement, "
								   + KEY_TITLE + " text not null,"
								   + KEY_DATE + " text not null,"
								   + KEY_LINK + " text not null,"
								   + KEY_LINK_MORE + " text not null,"
								   + KEY_IMAGE + " text);";
		}
	}
	
	// Constructor for the DatabaseLayer
	public DatabaseLayer(Context mCtx) {
		this.mCtx = mCtx;
	}
	
	// Open a connection to the database
	public DatabaseLayer open() throws 	SQLException {
		if (!dbOpen) {
			dbHelper = new DatabaseHelper(mCtx);
			db = dbHelper.getWritableDatabase();
			dbOpen = true;
		}
		return this;
	}
	
	// Close the connection
	public void close() {
		if (dbOpen) {
			dbHelper.close();
			dbOpen = false;
		}
	}
	
	// Upgrade databases
	public void upgrade() {
		dbHelper.onUpgrade(db, 1, 2);
	}
	
	private class fetchVals extends AsyncTask<String, Void, Cursor> {
		@Override
		protected Cursor doInBackground(String... params) {
			return db.query(params[0], new String[] { KEY_TITLE, KEY_DATE, KEY_LINK, KEY_LINK_MORE, KEY_IMAGE, KEY_ROWID },
					params[1], null, null, null, null);
		}
	}
	
	// Get all the rows from the news table
	public Cursor fetchAll(String table) {
		try {
			return new fetchVals().execute(table, null).get();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	// Get all the rows from the news table
	public List<SavedItem> fetchAll(String table, int type) {
		try {
			// Declare Variables
			List<SavedItem> savedItemList = new ArrayList<SavedItem>();
			
			// Set from URL
			switch (type) {
			case ServiceInfo.READ_NEWS:
				break;
			case ServiceInfo.TWITTER:
				break;
			case ServiceInfo.BLOG:
				break;
			case ServiceInfo.YOUTUBE:
				break;
			}

			// Open the database for writing
			open();
			
			Cursor cursor = new fetchVals().execute(table, null).get();
	
			if (cursor != null && cursor.moveToFirst()) {

				do {
					/*
					 * Title: getString(0)
					 * Date: getString(1)
					 * Link: getString(2)
					 * More: getString(3)
					 * Image: getString(4)
					 * _id in SQLite DB: getLong(5)
					 */
					savedItemList.add(new SavedItem(cursor.getString(0), cursor.getString(1), cursor.getString(2), cursor.getString(3), cursor.getLong(5), cursor.getString(4)));

				} while (cursor.moveToNext());
				
				if (cursor != null && !cursor.isClosed()) {
					cursor.close();
				}
			}
			
			// close the database
			close();
			
			return savedItemList;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	// Fetch a single item, for an item based on a position: whereClause = KEY_ROWID+"="+rowId
	public Cursor fetchItem(String table, String whereClause) throws SQLException {
		try {
			Cursor ret = new fetchVals().execute(table, whereClause).get();
			
			if (ret != null) {
				ret.moveToFirst();
			}
			return ret;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	// Add a news item
	public boolean addItem(String table, String title, String date, String link, String readMore, String image) {
		// Check to make sure the item is not already added
		
		if (fetchItem(table, KEY_TITLE+"=\""+title+"\"").getCount() < 1) {
			new insertVals().execute(table, title, date, link, readMore, image);
			return true;
		}
		return false;
	}
	
	private class insertVals extends AsyncTask<String, Void, Void> {
		@Override
		protected Void doInBackground(String... params) {
			ContentValues vals = new ContentValues();
			vals.put(KEY_TITLE, params[1]);
			vals.put(KEY_DATE, params[2]);
			vals.put(KEY_LINK, params[3]);
			vals.put(KEY_LINK_MORE, params[4]);
			vals.put(KEY_IMAGE, params[5]);
			
			if (!DatabaseLayer.this.db.isOpen());
			{
				DatabaseLayer.this.open();
			}
			db.insertOrThrow(params[0], null, vals);
			return null;
		}		
	}
	
	private class deleteVals extends AsyncTask<String, Void, Boolean> {
		@Override
		protected Boolean doInBackground(String... params) {
			db.beginTransaction();
			long nRows = 0;
			try {
				nRows = db.delete(params[0], KEY_ROWID + "=" + params[1], null);
				db.setTransactionSuccessful(); // So it commits the data when endTransaction() is called
			} catch (SQLException dbException) { 
				dbException.printStackTrace();
				//throw dbException;
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				db.endTransaction();
			}
			return nRows > 0;
		}
	}
	
	// Delete an entry on request
	public boolean deleteItem(String table, Long rowId) {
		try {
			//System.out.println("All clear: " + rowId.toString());
			return new deleteVals().execute(table, rowId.toString()).get();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} 
	}
}
